<?php

abstract class VSCAN_Model_Abstract
{
    /**
     * @var     int
     */
    protected $_id;
    /**
     * @var     VSCAN_Model_Mapper_Abstract
     */
    protected $_mapper;
    /**
     * @var     string
     */
    protected $_created;

    /**
     * @var     string
     */
    protected $_modified;
    /**
     * Constructor method for this data model to populate the model with given
     * data elements.
     * 
     * @param   array|Zend_Db_Table_Row $params
     */
    public function __construct($params = null)
    {
        if (null !== $params) {
            $this->populate($params);
        }
    }
    
	public function __set($name, $value)
    {
        $method = 'set' . ucfirst($name);
        if (!method_exists($this, $method)) {
        	require_once 'Zend/Exception.php';
            throw new Zend_Exception('Invalid property');
        }
        $this->$method($value);
    }
 
    public function __get($name)
    {
        $method = 'get' . ucfirst($name);
        if (!method_exists($this, $method)) {
        	require_once 'Zend/Exception.php';
            throw new Zend_Exception('Invalid property');
        }
        return $this->$method();
    }
    
    public function setId($id)
    {
        $this->_id = (int) $id;
        return $this;
    }
    public function getId()
    {
        return $this->_id;
    }
    public function setCreated($created)
    {
        $this->_created = $created;
    }
    public function getCreated()
    {
        return $this->_created;
    }
    public function setModified($modified)
    {
        $this->_modified = $modified;
    }
    public function getModified()
    {
        return $this->_modified;
    }
    
    /**
     * Sets the data mapper class for this model
     * 
     * @param   string|VSCAN_Model_Mapper_Abstract $mapper
     * @throws  Zend_Exception (for testing purposes only)
     */
    public function setMapper($mapper)
    {
        if (is_string($mapper)) {
            if (!class_exists($mapper)) {
                require_once 'Zend/Exception.php';
                throw new Zend_Exception('Non-existing mapper class provided');
            }
            $mapper = new $mapper;
        }
        if (!$mapper instanceof VSCAN_Model_Mapper_Abstract) {
            require_once 'Zend/Exception.php';
            throw new Zend_Exception('Invalid mapper class provided');
        }
        $this->_mapper = $mapper;
    }
    
    /**
     * Method to retrieve the mapper instance so it can be used within our data
     * model without having to instantiate it.
     * 
     * @return  VSCAN_Model_Mapper_Abstract
     */
    abstract public function getMapper();
    
    /**
     * Populate this data model with given data elements
     * 
     * @param   array|Zend_Db_Table_Row $row
     * @return  VSCAN_Model_Abstract
     */
 	public function populate($row)
    {
        if (is_array($row)) {
            $row = new ArrayObject($row, ArrayObject::ARRAY_AS_PROPS);
        }

    	$methods = get_class_methods($this);
        foreach ($row as $key => $value) {
            $method = 'set' . ucfirst($key);
            if (in_array($method, $methods)) {
                $this->$method($value);
            }
        }
        return $this;
    }
    
    /**
     * Converts this model into an array for storage or rendering purposes
     * 
     * @uses required for sub-class
     * 
     * @return  array
     */
    abstract public function toArray();
    
    /**
     * Finds a single data set matching given primary key ID
     * 
     * @uses required for sub-class
     * 
     * @param   int| $id
     */
    public function find($id)
    {
    	$this->getMapper()->find($this, $id);
    	return $this;
    }
    
    /**
     * Fetch all entries matching given conditions
     * 
     * @param   string|array|null $where
     * @param   string|array|null $order
     * @param   int|null $count
     * @param   int|null $offset
     * @return  array
     */
    public function fetchAll($where = null, $order = null, $count = null, $offset = null)
    {
        $className = get_class($this);
        return $this->getMapper()->fetchAll($className, $where, $order, $count, $offset);
    }
    
	/**
     * Fetch all for page
     * 
     * @param   string $className
     * @param   string|array $where
     * @param   string|array $order
     * @param   int|null $count
     * @param   int|null $offset
     * @return  array
     * @throws  Zend_Exception (for example purposes)
     */
    public function fetchAllForPage($select)
    {
        $page = Zend_Controller_Front::getInstance()->getRequest()->getParam('page', 1);
        if ($page === "all") {
            //hack for all pages at the same time
            $itemsPerPage = null;
        } else {
            $itemsPerPage = Zend_Registry::get('config')->pagination->itemsPerPage;
        }
        $query = $this->select();
    	$paginator = new Zend_Paginator(
            new Zend_Paginator_Adapter_DbTableSelect($select)
    	);
    	$paginator->setItemCountPerPage($itemsPerPage);
    	$paginator->setCurrentPageNumber($page);
    	$rows = $paginator->getCurrentItems();
    	$className = get_class($this);
    	$items = array();
    	foreach ($rows as $row) { 
    		$model = new $className;
    		$model->populate($row);
    		$this->getMapper()->_mapRelationship($model, $row); 
            $items[] = $model;
            unset ($model);
    	}
    	$paginator->setCurrentItems($items);
    	return $paginator;
    }
    
	/**
     * Fetch all active entries matching given conditions
     * 
     * @param   string|array|null $order
     * @param   int|null $count
     * @param   int|null $offset
     * @return  array
     */
    public function fetchAllActive($where = null, $order = null, $count = null, $offset = null)
    {
    	if (!empty($where)) {
    		$where = 'active = 1 and ' . $where;
    	} else {
    		$where = 'active = 1';
    	}
    	
        return $this->fetchAll($where, $order, $count, $offset);
    }
    
    /**
     * Find an single instance of this model matching given conditions
     * 
     * @param   string|array|null $where
     * @param   string|array|null $order
     */
    public function fetchRow($where = null, $order = null)
    {
        $this->getMapper()->fetchRow($this, $where, $order);
        return $this;
    }
    
    /**
     * Saves this model to the datastorage
     * 
     * @param   string $primaryKey
     * @return  null|int The last sequence or affected rows by this action
     */
    public function save($primaryKey = 'id')
    {
        return $this->getMapper()->save($this, $primaryKey);
    }
    
	/**
     * Delete this mode to the datastorage
     * 
     * @param   string $primaryKey
     * @return  null|int The last sequence or affected rows by this action
     */
    public function delete($primaryKey = 'id')
    {
        return $this->getMapper()->delete($this, $primaryKey);
    }
    
    /**
     * 
     *  Returns an instance of a Zend_Db_Table_Select object.
     *  @return Zend_Db_Table_Select
     */
	public function select()
    {
        return $this->getMapper()->select();
    }
    /**
     * Is model is mapped
     * @return boolean
     */
    public function isMapped()
    {
    	$id = $this->getId();
    	return !empty($id);
    } 
    /**
     * Convert a given value to aModel if is needed
     * @param string $className
     * @param VSCAN_Model_Abstract|int $value
     */
    protected function _convertToModel($className, &$value){

    	if (!class_exists($className)){
    		require_once 'Zend/Exception.php';
        	throw new Zend_Exception('class provided not exists');
    	}
    	if ((!$value instanceof $className) && 0 < (int)$value){
    		$valueObj = new $className();
    		$valueObj->find((int)$value);
    		$value = $valueObj;
    	} else if ($value instanceof $className) {
    		$value->find($value->getId());
    	}
    }
}